// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.key.polytomous;

import org.eclipse.jface.viewers.StyledCellLabelProvider;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.StyledString.Styler;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.TextStyle;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.description.KeyStatement;
import eu.etaxonomy.cdm.model.description.PolytomousKeyNode;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Apr 18, 2011
 * @version 1.0
 */
public class PolytomousKeyListLabelProvider extends StyledCellLabelProvider {

	private static Color colorLink = Display.getCurrent().getSystemColor(
			SWT.COLOR_BLUE);
	private Styler styler;

	private static final String EMPTY = "";
	// TODO make this configurable via preferences
	private static final String INCREMENTOR_CHARACTER = "'";

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.viewers.StyledCellLabelProvider#update(org.eclipse.
	 * jface.viewers.ViewerCell)
	 */
	@Override
	public void update(ViewerCell cell) {
		Object element = cell.getElement();
		int columnIndex = cell.getColumnIndex();

		if (element instanceof PolytomousKeyNode) {

			String text = getTextForColumnIndex((PolytomousKeyNode) element,
					columnIndex);
			
			
			cell.setText(text);

			if (columnIndex == 4 || columnIndex == 5) {
				StyledString styledString = new StyledString(text, getStyler());
				StyleRange[] styleRanges;
				styleRanges = styledString.getStyleRanges();
				cell.setStyleRanges(styleRanges);
			}
		}

		super.update(cell);
	}

	private String getTextForColumnIndex(PolytomousKeyNode node, int columnIndex) {
		switch (columnIndex) {
		case 0:
			return getItemNumber(node);
		case 1:
			return getItemQuestion(node);			
		case 2:
			return getItemEdgeNumber(node);
		case 3:
			return getItemStatement(node);
		case 4:
			return getItemLink(node);
		case 5:
            return getItemTaxon(node);
		}
		return EMPTY;
	}

	/**
	 * @return
	 */
	private Styler getStyler() {
		if (styler == null) {
			styler = new Styler() {
				@Override
				public void applyStyles(TextStyle textStyle) {
					textStyle.underline = true;
					textStyle.underlineColor = colorLink;
					textStyle.foreground = colorLink;
					textStyle.underlineStyle = SWT.UNDERLINE_SINGLE;
				}
			};
		}
		return styler;
	}

	private String getItemNumber(PolytomousKeyNode node) {
		if (isParentRoot(node)) {
			return "root";
		} else {
			
			String itemNumber = (node.getParent().getNodeNumber() != null) ? node.getParent().getNodeNumber().toString() : EMPTY;					
			return itemNumber;
		}
	}

	private String getItemEdgeNumber(PolytomousKeyNode node) {
		String itemEdgeNumber = node.getParent().getNodeNumber() != null ? node.getParent().getNodeNumber().toString() : EMPTY;		
		PolytomousKeyNode parent = getParent(node);
		int index = parent.getChildren().indexOf(node);

		char numberChar = 'a';
		//FIXME: Currently this numbering works only until 'z', after which all siblings will be named with 'z'
		for (int i = 0; i < index; i++) {
			if(index < 26) {
				numberChar++;
			} else {
				numberChar = 'z';
			}
			//itemEdgeNumber += INCREMENTOR_CHARACTER;
		}
		return itemEdgeNumber + numberChar;
	}
	
	private String getItemQuestion(PolytomousKeyNode node) {
		if (isParentRoot(node)) {
			return "";
		} else {
			KeyStatement question = getParent(node).getQuestion();					
			return question != null ? CdmUtils.Nz(question.getLabelText(CdmStore.getDefaultLanguage())) : EMPTY;
			
		}

	}

	private String getItemStatement(PolytomousKeyNode node) {
		KeyStatement statement = node.getStatement();
		return statement != null ? CdmUtils.Nz(statement.getLabelText(CdmStore
				.getDefaultLanguage())) : EMPTY;
	}

	private String getItemLink(PolytomousKeyNode node) {
		String linkString = node.getChildren().isEmpty() ? EMPTY : node.getNodeNumber().toString();

		return linkString;
	}
	
	private String getItemTaxon(PolytomousKeyNode node) {
	    String taxonString = node.getTaxon() != null ? node.getTaxon().getName().getTitleCache() : EMPTY;

	    return taxonString;
	}

	private PolytomousKeyNode getParent(PolytomousKeyNode node) {
		return node.getParent();
	}

	private boolean isParentRoot(PolytomousKeyNode node) {
		return getParent(node) == null;
	}

}
