/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.operation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.api.application.ICdmApplicationConfiguration;
import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.api.service.config.TaxonDeletionConfigurator;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.operation.AbstractPostTaxonOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>DeleteMisapplicationOperation class.</p>
 *
 * @author p.ciardelli
 * @created 16.01.2009
 * @version 1.0
 */
public class DeleteMisapplicationOperation extends AbstractPostTaxonOperation {

	private final Taxon misapplication;

	private Reference<?> citation;

	private String microcitation;

	/**
	 * <p>Constructor for DeleteMisapplicationOperation.</p>
	 *
	 * @param label a {@link java.lang.String} object.
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param taxon a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 * @param misapplication a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 */
	public DeleteMisapplicationOperation(String label, IUndoContext undoContext,
			Taxon taxon, Taxon misapplication, IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, taxon, postOperationEnabled);

		this.misapplication = misapplication;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		// Find misapplication relation, save citation information
		for (TaxonRelationship relationship : element.getTaxonRelations()) {
			if (relationship.getType().equals(TaxonRelationshipType.MISAPPLIED_NAME_FOR())
					&& relationship.getFromTaxon().equals(misapplication)) {
				citation = relationship.getCitation();
				microcitation = relationship.getCitationMicroReference();
			}
		}
		monitor.worked(20);

		// Remove misapplied name relation from taxon
		element.removeTaxon(misapplication, TaxonRelationshipType.MISAPPLIED_NAME_FOR());
		ICdmApplicationConfiguration controller;

		controller = (ICdmApplicationConfiguration) CdmStore.getCurrentApplicationConfiguration();

		ITaxonService service = controller.getTaxonService();
		TaxonDeletionConfigurator config = new TaxonDeletionConfigurator();
		
		service.deleteTaxon(misapplication, config, null);
		monitor.worked(40);

		return postExecute(null);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		element.addMisappliedName(misapplication, citation, microcitation);

		return postExecute(misapplication);
	}
}
