package eu.etaxonomy.taxeditor.editor.name.operation;

import java.util.Iterator;
import java.util.Set;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.ui.IWorkbenchPage;

import eu.etaxonomy.cdm.api.application.ICdmApplicationConfiguration;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.api.service.DeleteResult;
import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.api.service.config.TaxonDeletionConfigurator;
import eu.etaxonomy.cdm.model.taxon.Classification;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

public class DeleteTaxonOperation extends DeleteTaxonBaseOperation{
	//private Taxon taxon;
	private final Classification classification;




	public DeleteTaxonOperation(String label, IUndoContext undoContext,
			Taxon taxon, Classification classification, IWorkbenchPage activePage, IPostOperationEnabled postOperationEnabled, IConversationEnabled conversationEnabled) {
		super(label, undoContext, taxon, activePage, postOperationEnabled, conversationEnabled);
		this.element = taxon;

		Set<TaxonNode> nodes = taxon.getTaxonNodes();
		if (nodes.size() == 1 && classification == null){
			this.taxonNode = nodes.iterator().next();
		} else if (classification != null){
			Iterator<TaxonNode> iterator = nodes.iterator();
			while (iterator.hasNext()){
				TaxonNode node = iterator.next();
				if (node.getClassification().equals(classification)){
					this.taxonNode = node;
				}
			}
		} else{
			//TODO
		}
		this.classification = classification;

	}



	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

				monitor.worked(20);
				bind();


				ICdmApplicationConfiguration controller;

				controller = (ICdmApplicationConfiguration) CdmStore.getCurrentApplicationConfiguration();

				ITaxonService service = controller.getTaxonService();
				

				DeleteResult result =	service.deleteTaxon(element, new TaxonDeletionConfigurator(), classification);


				//closeObsoleteEditor(taxon);
				monitor.worked(40);


				return postExecute(null);
	}

	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO Auto-generated method stub
		return null;
	}


}
