/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.view.concept.graph;

import java.util.Set;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.viewers.Viewer;
import org.eclipse.zest.core.viewers.IGraphContentProvider;



import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;

/**
 * @author andreas 
 *
 */
public class ConceptGraphContentProvider implements IGraphContentProvider {

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
	 */
	@Override
	public void dispose() {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
	 */
	@Override
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.eclipse.zest.core.viewers.IGraphContentProvider#getSource(java.lang.Object)
	 */
	@Override
	public Object getSource(Object taxonRelationship) {
		return ((TaxonRelationship)taxonRelationship).getFromTaxon();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.zest.core.viewers.IGraphContentProvider#getDestination(java.lang.Object)
	 */
	@Override
	public Object getDestination(Object taxonRelationship) {
		return ((TaxonRelationship)taxonRelationship).getToTaxon();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.zest.core.viewers.IGraphContentProvider#getElements(java.lang.Object)
	 */
	@Override
	public Object[] getElements(Object inputElement) {
		if(inputElement instanceof TaxonEditorInput){
			Taxon taxon = ((TaxonEditorInput) inputElement).getTaxon();

			Map<String, TaxonRelationship> filteredTaxonRelations = new HashMap<String, TaxonRelationship>();
			getTaxonRelationshipsRecursive(filteredTaxonRelations, taxon);
			
			return filteredTaxonRelations.values().toArray();
		}
		return new Object[0];
	}

	private void getTaxonRelationshipsRecursive(Map<String, TaxonRelationship> filteredTaxonRelations, Taxon taxon) {
		// TODO extract method into new class TaxonHelper in
		// eu.etaxonomy.taxeditor.model;
		// see also ConceptContentProvider
		for (TaxonRelationship relationship : taxon.getTaxonRelations()) {
			
			if (!relationship.getType().equals(TaxonRelationshipType.MISAPPLIED_NAME_FOR())
					&& !relationship.getType().equals(TaxonRelationshipType.INVALID_DESIGNATION_FOR())
					&& !relationship.getType().equals(TaxonRelationshipType.TAXONOMICALLY_INCLUDED_IN())) {
				
				if(!relationship.getToTaxon().equals(taxon)){
					continue;
				}
				
				String key = relationship.getFromTaxon().getUuid().toString() + relationship.getType().getUuid();
				if(!filteredTaxonRelations.containsKey(key)) {
					filteredTaxonRelations.put(key, relationship);					
					Taxon toTaxon = relationship.getToTaxon();
					if(toTaxon != null){
						getTaxonRelationshipsRecursive(filteredTaxonRelations, toTaxon);
	
					}
				}
			}
			
			
		}
		
		
	}

}
