package eu.etaxonomy.taxeditor.editor.view.dataimport;

import java.io.IOException;

import org.apache.http.client.ClientProtocolException;
import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.ContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.CompoundContributionItem;

import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.ext.occurrence.OccurenceQuery;
import eu.etaxonomy.cdm.ext.occurrence.gbif.DataSetResponse;
import eu.etaxonomy.cdm.ext.occurrence.gbif.GbifDataSetProtocol;
import eu.etaxonomy.cdm.ext.occurrence.gbif.GbifQueryServiceWrapper;
import eu.etaxonomy.cdm.ext.occurrence.gbif.GbifResponse;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

public class DataImportEditorContextMenu extends CompoundContributionItem {

    private final Logger logger = Logger.getLogger(DataImportEditorContextMenu.class);

    /* (non-Javadoc)
     * @see org.eclipse.ui.actions.CompoundContributionItem#getContributionItems()
     */
    @Override
    protected IContributionItem[] getContributionItems() {
        IContributionItem[] contributionItems = new IContributionItem[] {
                new ContributionItem() {
                    @Override
                    public void fill(Menu menu, int index) {
                        final IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
                        final IEditorPart activeEditor = window.getActivePage().getActiveEditor();
                        final ISelection selection = window.getActivePage().getSelection();
                        if(selection instanceof StructuredSelection){
                            final StructuredSelection structuredSelection = (StructuredSelection)selection;

                            //Save Specimen
                            MenuItem saveSpecimenItem = new MenuItem(menu, SWT.NONE);
                            saveSpecimenItem.setText("Save Specimen");
                            saveSpecimenItem.addListener(SWT.Selection, new Listener() {

                                @Override
                                public void handleEvent(Event event) {
                                    if(structuredSelection.iterator().hasNext()){
                                        Object next = structuredSelection.iterator().next();
                                        if(next instanceof GbifResponse){
                                            //unwrap DerivedUnit from GbifResponse
                                            next = ((GbifResponse) next).getDerivedUnitFacade().innerDerivedUnit();
                                        }
                                        if(next instanceof SpecimenOrObservationBase){
                                            if(activeEditor instanceof DataImportEditor){
                                                DataImportEditor<?> dataImportEditor = (DataImportEditor<?>)activeEditor;
                                                CdmStore.getService(IOccurrenceService.class).saveOrUpdate((SpecimenOrObservationBase<?>) next);
                                                dataImportEditor.getConversationHolder().commit(true);
                                            }
                                        }
                                    }
                                }
                            });

                            //Query original data provider
                            if(structuredSelection.size()==1
                                    && structuredSelection.iterator().next() instanceof GbifResponse
                                    && ((GbifResponse)structuredSelection.iterator().next()).getDataSetProtocol()!=null
                                    && ((GbifResponse)structuredSelection.iterator().next()).getDataSetProtocol()==GbifDataSetProtocol.BIOCASE){
                                final GbifResponse gbifResponse = (GbifResponse)structuredSelection.iterator().next();
                                MenuItem queryOriginalItem = new MenuItem(menu, SWT.NONE);
                                queryOriginalItem.setText("Query original data provider");
                                queryOriginalItem.addListener(SWT.Selection, new Listener() {

                                    @Override
                                    public void handleEvent(Event event) {
                                        try{
                                            DataSetResponse response = new GbifQueryServiceWrapper().queryOriginalDataSet(gbifResponse);
                                            final BioCaseEditorInput input = new BioCaseEditorInput(new OccurenceQuery(response.getUnitId()), response.getEndpoint());
                                            EditorUtil.open(input);
                                            Job queryJob = new Job("Querying for original record...") {

                                                @Override
                                                protected IStatus run(IProgressMonitor monitor) {
                                                    input.query();
                                                    return Status.OK_STATUS;
                                                }
                                            };
                                            queryJob.schedule();
                                        } catch (ClientProtocolException e) {
                                            logger.error("Query for single record failed.", e);
                                        } catch (IOException e) {
                                            logger.error("Query for single record failed.", e);
                                        } catch (PartInitException e) {
                                            logger.error("Could not open editor.", e);
                                        }
                                    }
                                });
                            }
                        }
                    }
                }};
        return contributionItems;
    }

}
