// $Id$
/**
 * Copyright (C) 2014 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.editor.view.derivate;

import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.DragSourceEvent;

import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.model.AbstractUtility;

/**
 * @author pplitzner
 * @date 18.03.2014
 *
 */
public class DerivateDragListener extends DragSourceAdapter{

    private final DerivateView derivateView;

    /**
     * @param derivateView
     */
    public DerivateDragListener(DerivateView derivateView) {
        this.derivateView = derivateView;
    }

    /* (non-Javadoc)
     * @see org.eclipse.swt.dnd.DragSourceAdapter#dragSetData(org.eclipse.swt.dnd.DragSourceEvent)
     */
    @Override
    public void dragSetData(DragSourceEvent event) {
        if (LocalSelectionTransfer.getTransfer().isSupportedType(event.dataType)) {
            DerivateViewSelection derivateViewSelection = new DerivateViewSelection(derivateView.getViewer().getSelection(), derivateView);
            LocalSelectionTransfer.getTransfer().setSelection(derivateViewSelection);
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.swt.dnd.DragSourceAdapter#dragStart(org.eclipse.swt.dnd.DragSourceEvent)
     */
    @Override
    public void dragStart(DragSourceEvent event) {
        if(derivateView.isDirty()){
            event.doit = false;
            AbstractUtility.warningDialog("View has unsaved changes", this, "You need to save before performing this action");
            return;
        }
        ISelection selection = derivateView.getViewer().getSelection();
        if(EditorUtil.getTreeNodeOfSelection(selection)!=null){
            event.doit = true;
        }
        else{
            event.doit = false;
        }
    }
}
