/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.view.media.handler;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.media.ImageFile;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;
import eu.etaxonomy.taxeditor.editor.view.media.operation.RemoveImageFromDescriptionElementOperation;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;

/**
 * <p>DeleteMediaHandler class.</p>
 *
 * @author p.ciardelli
 * @version $Id: $
 */
public class DeleteMediaHandler extends AbstractHandler {

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	/** {@inheritDoc} */
	public Object execute(ExecutionEvent event) throws ExecutionException {
		IWorkbenchPart part = HandlerUtil.getActivePart(event);
		IPostOperationEnabled postOperationEnabled = (part instanceof IPostOperationEnabled) ? (IPostOperationEnabled) part : null;
				
		IEditorPart editor = HandlerUtil.getActiveEditor(event);
		if (editor instanceof FormEditor) {
			editor = ((FormEditor) editor).getActiveEditor();
		}
		IEditorInput input = editor.getEditorInput();
		if (input instanceof TaxonEditorInput) {
			Taxon taxon = ((TaxonEditorInput) input).getTaxon();
			
			ISelection selection = HandlerUtil.getCurrentSelection(event);
			if (selection instanceof TreeSelection) {
				TreePath[] paths = ((TreeSelection) selection).getPaths();
				
				int count = paths[0].getSegmentCount();
				DescriptionElementBase element = null;
				for (int i = 0; i < count; i++ ) {
					if (paths[0].getSegment(i) instanceof DescriptionElementBase) {
						element = (DescriptionElementBase) paths[0].getSegment(i);
						break;
					}
				}
				ImageFile image = (ImageFile) paths[0].getLastSegment();
				
				AbstractPostOperation operation = null;
				try {
					// TODO use undo context specific to editor
					operation = new RemoveImageFromDescriptionElementOperation(event.getCommand().getName(), 
							EditorUtil.getUndoContext(), taxon, image, element, postOperationEnabled);
					EditorUtil.executeOperation(operation);
				} catch (NotDefinedException e) {
					EditorUtil.warn(getClass(), "Command name not set.");
				}
			}
		}
		return null;
	}

}
